<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 *
 * @package    local_mb2builder
 * @copyright  2018 - 2024 Mariusz Boloz (lmsstyle.com)
 * @license    Commercial https://themeforest.net/licenses
 */

//defined('MOODLE_INTERNAL') || die();

require_once( __DIR__ . '/../../config.php' );
require_once( __DIR__ . '/lib.php' );
require_once( __DIR__ . '/classes/pages_api.php' );
require_once( __DIR__ . '/classes/pages_helper.php' );
require_once( $CFG->libdir . '/adminlib.php' );
require_once( $CFG->libdir . '/tablelib.php' );

// Optional parameters
$deleteid = optional_param('deleteid', 0, PARAM_INT);
$returnurl = optional_param('returnurl', '/local/mb2builder/index.php', PARAM_LOCALURL);
$page = optional_param('page', 0, PARAM_INT);
$perpage = 20;

// Links
$editpage = '/local/mb2builder/edit-page.php';
$managepages = '/local/mb2builder/index.php';
$deletepage = '/local/mb2builder/delete-page.php';
$baseurl = new moodle_url($managepages);
$returnurl = new moodle_url($returnurl);

// Configure the context of the page
$context = context_system::instance();
$PAGE->set_context( $context );
$PAGE->set_url( '/local/mb2builder/index.php' );
$PAGE->set_pagelayout('admin'); // This is require for page navigation

require_capability( 'local/mb2builder:managepages', context_system::instance() );
$can_manage = has_capability( 'local/mb2builder:managepages', context_system::instance() );

// Get sorted pages
$sortorder_items = Mb2builderPagesApi::get_list_records($page * $perpage, $perpage);
$countitems = Mb2builderPagesApi::get_list_records(0,0,true);

// Delete the page
if ($can_manage && $deleteid)
{
    Mb2builderPagesApi::delete($deleteid);
    $message = get_string('pagedeleted', 'local_mb2builder');
}

if (isset($message))
{
    redirect($returnurl, $message);
}

// Page title
$titlepage = get_string('pluginname', 'local_mb2builder');
$PAGE->set_heading($titlepage);
$PAGE->set_title($titlepage);
echo $OUTPUT->header();
echo $OUTPUT->heading(get_string('managepages', 'local_mb2builder'));

// Table declaration
$table = new flexible_table('mb2builder-pages-table');

// Customize the table
$table->define_columns(
    array(
        'name',
        'shortcode',
        'moodlepage',
        'actions',
    )
);

$table->define_headers(
    array(
        get_string('name', 'moodle'),
        get_string('shortcode', 'local_mb2builder'),
        get_string('moodlepage', 'local_mb2builder'),
        get_string('actions', 'moodle'),
    )
);

$table->define_baseurl($baseurl);
$table->set_attribute( 'class', 'generaltable' );
$table->column_class( 'actions', 'text-right align-middle' );
$table->column_class( 'name', 'align-middle' );
$table->column_class( 'shortcode', 'align-middle' );
$table->setup();

foreach ($sortorder_items as $item)
{    

	// Filling of information columns
    $titlecallback = html_writer::div(html_writer::link(new moodle_url($editpage, array('itemid' => $item->id)), '<strong>' . $item->title . '</strong>'));

    // Created and modified by
    $shortcodeitem = '<div><input style="width:auto;min-width:148px;min-height:0;padding:.12rem .2rem;border-color:#dee2e6;line-height:1;color:green!important;font-weight:bold;" type="text" value=\'[mb2page pageid="' . $item->id . '"]\' readonly="readonly" onfocus="this.select()" /></div>';

    // Used in
    $checkforpage = Mb2builderPagesApi::check_for_page( $item->id );
    $moodlepageitem = $checkforpage ? $checkforpage : get_string( 'nopage', 'local_mb2builder' );

    // Link for editing
    $editlink = new moodle_url($editpage, array('itemid' => $item->id));
    $edititem = $OUTPUT->action_icon($editlink, new pix_icon('t/edit', get_string('edit', 'moodle')));

    // Link to remove
    $deletelink = new moodle_url($deletepage, array('deleteid' => $item->id));
    $deleteitem = $OUTPUT->action_icon($deletelink, new pix_icon('t/delete', get_string('delete', 'moodle')));

    // Check if user can manage items
    $actions = $can_manage ? $edititem . $deleteitem : '';

	$table->add_data( array( $titlecallback, $shortcodeitem, $moodlepageitem, $actions ) );
    //$table->add_data( array( $titlecallback, $moodlepageitem, $actions ) );

}

// Display the table
$table->print_html();

echo $OUTPUT->paging_bar( $countitems, $page, $perpage, $baseurl );

echo $OUTPUT->footer();
?>
